/*
 * u3cheat11.c
 *
 * Main Engine for the Ultima III Character Editor.
 *
 * Copyright  1995 by Michael D. Trent; all rights reserved
 */

#include "alerts11.h"
#include "datatypes11.h"
#include "dlogutil.h"
#include "DSCH.h"
#include "ResIDs.h"
#include "roster11.h"
#include "mystring.h"


#define kNamStrLen	13 /* 12 + 1 */

#define kEnterKey		3
#define kReturnKey		13
#define kEscapeKey		27
#define kLeftKey		28
#define kRightKey		29
#define kUpKey			30
#define kDownKey		31

FSSpec spec;
short AQty[16];
short WQty[16];
Boolean dirty = false;

void WitchChant(void);
Boolean GetFile(FSSpec *spec);
Boolean DoRosterDialog();
void SetUpRosterDialog(DialogPtr d);
pascal Boolean RosterFilterProc(DialogPtr d, EventRecord *e, short *itemHit);
short FindActiveRadio(DialogPtr d);
short FindControlsIdx(DialogPtr d, Handle c);
void SetRadio(DialogPtr d, short item);
void DoEditDialog(short id);
void SetUpEditDialog(DialogPtr d,short id);
void CopyEditDialog(DialogPtr d,short id);
void HiliteRosterControls(DialogPtr d, short val);
void SetNearestRadio(DialogPtr d, short item, short countBy,short hOrV);
pascal Boolean EditFilterProc(DialogPtr d, EventRecord *e, short *itemHit);
Boolean ValidateEditDialog(DialogPtr d);


main()
{
	OSErr err;

	WitchChant();
	
	MaxApplZone();
	MoreMasters();	MoreMasters();	MoreMasters();	MoreMasters();
	
	err = LoadDSCHList();
	if (err != noErr) {
		DoError("\pInternal Resource Error!",err,kFatal);
	}
	
	if (GetFile(&spec)) {
		err = LoadRoster(&spec);
		if (err == noErr) {
			DoRosterDialog();
		} else {
			DoError("\pError loading Roster file!",err,false);
		}
	}
} 

void WitchChant(void)
{
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
}

Boolean GetFile(FSSpec *spec)
{
	SFTypeList typeList;
	StandardFileReply reply;
	
	typeList[0] = 'RSTR';
	StandardGetFile(nil, 1, typeList, &reply);
	
	*spec = reply.sfFile;
	return (reply.sfGood);
}

Boolean DoRosterDialog()
{
	DialogPtr d;
	Boolean done = false;
	short itemHit,ync,id;
	OSErr err;
	
	
	d = GetNewDialog(130,0L,(WindowPtr)-1);
	if (!d) DoError("\pCan't Load Dialog!!",0,true);

	SetUpRosterDialog(d);
	//(void) SetDialogDefaultItem(d, 1); // 1.1.1 fix 10/21/95

	ShowWindow(d);
	SetPort(d);		
	DrawDefaultButton(d);
	
	while (!done) {
		DrawDefaultButton(d);
		
		// MDT 5/5/96
		{
		short iType;
		Handle iHandle;
		Rect iRect;
		GetDItem(d, 2, &iType, &iHandle, &iRect);   /* IM I pg 421 */
		if (dirty) {
			HiliteControl((ControlHandle)iHandle, 0);
		} else {
			HiliteControl((ControlHandle)iHandle, 255);
		}
		}
		
		ModalDialog(RosterFilterProc,&itemHit);
		
		switch (itemHit) {
		case 1:
			id = FindActiveRadio(d) - 4;
			HiliteRosterControls(d,255);
			DoEditDialog(id);
			HiliteRosterControls(d,0);

			LoadRadioControl(d,id);
			break;
		case 3:
			if (!dirty) {
				ync=3;
			} else {
				ync = SaveChanges();
			}
			if (ync == 2) break;// cancel
			done = true;
			if (ync == 3) break; // don't save
			// fall through - save
		case 2:
			err = SaveRoster(&spec);
			if (err != noErr) {
				DoError("\pError saving Roster file!",err,false);
			}
			dirty=false;
			break;
		default:
			;
		}
	}
	DisposeDialog(d);
}

void SetUpRosterDialog(DialogPtr d)
{
	short id;

	short	iType;
	Handle	iHandle;
	Rect	iRect;
	
	for (id=0; id<20;id++) {
		LoadRadioControl(d,id);
	}	
	
	// prime our radio buttons.
	SetNearestRadio(d,4,1,0);
	if (FindActiveRadio(d) < 0) {
		GetDItem(d,1,&iType,&iHandle,&iRect);
		HiliteControl((ControlHandle)iHandle,255); //(dim control)
		GetDItem(d,2,&iType,&iHandle,&iRect);
		HiliteControl((ControlHandle)iHandle,255); //(dim control)
	}
}


pascal Boolean RosterFilterProc(DialogPtr d, EventRecord *e, short *itemHit)
{
	char 	theChar;
	short	i, iType;
	Handle	iHandle;
	Rect	iRect;
	short 	radio;
	Point	where;
	OSErr err;
	
	ModalFilterUPP	stdFilterProc;
	Boolean ret;

	unsigned long DoubleTime = GetDblTime();	

	static	long	lastWhen = 0;
	static	short	lastControl=0;	
	
	switch(e->what) {
	case keyDown:
	case autoKey:		
		theChar = (e->message & charCodeMask);
		
		switch(theChar){
		case kReturnKey:
		case kEnterKey:
		case 'e':
		case 'E':
			GetDItem(d, 1, &iType, &iHandle, &iRect);
			if ((*(ControlHandle)iHandle)->contrlHilite == 255) break;
			
			FlashButton((ControlHandle)iHandle);
			*itemHit = 1;
			return (true);
			break; //unreachable code
		case '.':
			if (!(e->modifiers & cmdKey))
				break;
			// else Fall Through!	
		case kEscapeKey:
		case 'Q':
		case 'q':
			GetDItem(d, 3, &iType, &iHandle, &iRect);
			if (iType == ctrlItem + btnCtrl) {		// If there is a Button
				FlashButton((ControlHandle)iHandle);
			
				*itemHit = 3;
				return (true);
			//} else {
			//	return (false);
			}
			break; // duh!!
		case 's':
		case 'S':
			GetDItem(d, 2, &iType, &iHandle, &iRect);
			if ((*(ControlHandle)iHandle)->contrlHilite == 255) break;
			FlashButton((ControlHandle)iHandle);
		
			*itemHit = 2;
			return (true);
			break; // duh!!
		case kUpKey:
			radio = FindActiveRadio(d) - 1;
			if (radio < 1) break;
			
			if (radio < 4) radio = 23;
			//SetRadio(d,radio);
			SetNearestRadio(d,radio,-1,0);
			break;
		case kDownKey:
			radio = FindActiveRadio(d) + 1;
			if (radio < 1) break;
			if (radio > 23) radio = 4;
			//SetRadio(d,radio);
			SetNearestRadio(d,radio,1,0);
			break;
		case kLeftKey:
			radio=FindActiveRadio(d);
			if (radio < 1) break;
			if (radio>13) {
				radio -= 10;
			} else {
				if (radio == 4) {
					radio = 23;
				} else {
					radio += 9;
				}
			}
			//SetRadio(d,radio);
			SetNearestRadio(d,radio,-1,1);			
			break;
		case kRightKey:
			radio=FindActiveRadio(d);
			if (radio < 1) break;
			if (radio<14) {
				radio += 10;
			} else {
				if (radio == 23) {
					radio = 4;
				} else {
					radio -= 9;
				}
			}
			//SetRadio(d,radio);
			SetNearestRadio(d,radio,1,1);
			break;
		default:
			;
		} // switch (theChar){
	break;
	case mouseDown:
		where = e->where;
		GlobalToLocal(&where);
		if (FindControl(where,d,(ControlHandle*)(&iHandle))) {	// is control?
			if (i = FindControlsIdx(d,iHandle)) {				// is radio control?
				if (TrackControl((ControlHandle)iHandle, where, nil)) { // good hit?
					if ( ( (e->when - lastWhen) < DoubleTime ) 
						&& (i == lastControl)) {		// is double click?
						// DoubleClick
						*itemHit = 1;
						return (true);
					} // if e->when
					lastWhen = e->when;					// is single click.
					lastControl = i;
				
					SetRadio(d,i);			// set Radio Control.
					e->what = nullEvent; 	// abort event
					return (false);
				} // if TrackControl
			} // if FindControlsIdx
		} // if FindControl
		break;
	case updateEvt: // MDT+2 5/5/96
		DrawDefaultButton(d);
		break;
	default:
		;
	}	// switch (e->what)
	//return (false);
	err = GetStdFilterProc(&stdFilterProc);
	if (err == noErr) ret = (stdFilterProc)(d,e,itemHit);
	else ret = false;
	
	return ret;
}

void HiliteRosterControls(DialogPtr d, short val)
{
	short i,iType;
	Handle iHandle;
	Rect iRect;
	
	for (i=1;i<4;i++) {
		GetDItem(d,i,&iType,&iHandle,&iRect);
		HiliteControl((ControlHandle)iHandle,val);
	}
}

short FindActiveRadio(DialogPtr d)
{
	short i,iType;
	Handle iHandle;
	Rect iRect;
	
	for (i=4;i<=23;i++) {
		GetDItem(d,i,&iType,&iHandle,&iRect);
		if (GetCtlValue((ControlHandle)iHandle) == 1)
			break;
	}
	if (i == 24) i = -1;
	
	return i;
}

short FindControlsIdx(DialogPtr d, Handle c)
{
	short i,iType;
	Handle iHandle;
	Rect iRect;
	
	for (i=4;i<=23;i++) {
		GetDItem(d,i,&iType,&iHandle,&iRect);
		if (c == iHandle)
			break;
	}
	if (i == 24) i = 0;
	
	return i;
}

void SetNearestRadio(DialogPtr d, short item, short countBy,short horv)
{
	short iType,c,realItem;
	Handle iHandle;
	Rect iRect;

	if (horv == 0) { // if Vertical
		realItem = item;
		for (c=0;c < 20;c++) {
			GetDItem(d,realItem,&iType,&iHandle,&iRect);
			if ((*(ControlHandle)iHandle)->contrlHilite == 255) {
				realItem = realItem + countBy;
				if (realItem > 23) realItem = 4;
				if (realItem < 4) realItem = 23;
			} else {
				break;
			}
		}
	} else { // if Horizontal
		realItem = item;
		for (c=0;c < 20;c++) {
			GetDItem(d,realItem,&iType,&iHandle,&iRect);
			if ((*(ControlHandle)iHandle)->contrlHilite == 255) {
				if (countBy<0) { // going left
					if (realItem>13) {
						realItem -= 10;
					} else {
						if (realItem == 4) {
							realItem = 23;
						} else {
							realItem += 9;
						}
					}
				} else { //going right 
					if (realItem<14) {
						realItem += 10;
					} else {
						if (realItem == 23) {
							realItem = 4;
						} else {
							realItem -= 9;
						}
					}
				}
			} else {
				break;
			}
		}
	}
	if (c == 20) realItem = 0; // set NONE!!

	SetRadio(d,realItem);
}

void SetRadio(DialogPtr d, short item)
{
	short i,iType,val;
	Handle iHandle;
	Rect iRect;

	for (i=4;i<=23;i++) {
		GetDItem(d,i,&iType,&iHandle,&iRect);
		val = (i == item) ? 1 : 0;
		SetCtlValue((ControlHandle)iHandle, val);
	}
}

void DoEditDialog(short id)
{
	DialogPtr d;
	GrafPtr	savePort;
	short itemHit;
	Boolean done = false;
	short lastArmor = 0;
	short lastWeapon = 0;
	short iType;
	Handle iQty,iHandle;
	Rect iRect;
	Str255 str;
	long num;
	
	GetPort(&savePort);
	
	d = GetNewDialog(131,0L,(WindowPtr)-1L);
	if (!d) DoError("\pCan't Load Dialog!!",0,true);
	
	ShowWindow(d);
	SetPort(d);
	
	SetUpEditDialog(d,id);
	
	(void)SetDialogDefaultItem(d,1);
	(void)SetDialogCancelItem(d,2);
	(void)SetDialogTrackCursor(d, true);	
	
	while (!done) {
		DrawDefaultButton(d);
		
		ModalDialog(EditFilterProc,&itemHit);
		
		switch (itemHit) {
		case 1:
			GetDItem(d,kName,&iType,&iHandle,&iRect);
			GetIText(iHandle,str);
			if (str[0] == 0) {
				BadName();
				break;
			} else {
				CopyEditDialog(d,id);
				dirty = true;
			}
			// fall through
		case 2:
			done = true;
			break;
		case kArmor:
			GetDItem(d,kAQty,&iType,&iQty,&iRect);
			GetIText(iQty,str);
			StringToNum(str,&num);
			AQty[lastArmor] = (short) num;
			
			GetDItem(d,kArmor,&iType,&iHandle, &iRect);
			lastArmor = GetCtlValue((ControlHandle)iHandle)-1;
			
			NumToString((long)AQty[lastArmor],str);
			SetIText(iQty,str);
			break;
		case kWeapons:
			GetDItem(d,kWQty,&iType,&iQty,&iRect);
			GetIText(iQty,str);
			StringToNum(str,&num);
			WQty[lastWeapon] = (short) num;
			
			GetDItem(d,kWeapons,&iType,&iHandle, &iRect);
			lastWeapon = GetCtlValue((ControlHandle)iHandle)-1;
			
			NumToString((long)WQty[lastWeapon],str);
			SetIText(iQty,str);
			break;
		case kLove:
		case kSol:
		case kMoons:
		case kDeath:
		case kForce:
		case kFire:
		case kSnake:
		case kKings:
			GetDItem(d,itemHit,&iType,&iHandle,&iRect);
			SetCtlValue((ControlHandle)iHandle,!GetCtlValue((ControlHandle)iHandle));
			break;
		default:
			//DebugStr("\pDoEditDialog Error: Bad itemHit!");
			; // some things aren't responded to.
		} // switch
	} // while
	
	SetPort(savePort);
	DisposeDialog(d);
}

pascal Boolean EditFilterProc(DialogPtr d, EventRecord *e, short *itemHit)
{
	short	item;
	DialogRecord *dr;
	OSErr err;
	short i;
	char c;
	
	short iType;
	Handle iHandle;
	Rect iRect;
	Str255 iText;
	
	ModalFilterUPP	stdFilterProc;
	Boolean ret;
	
	unsigned long size,maxSize;
	
	switch(e->what) {
	case keyDown:
	case autoKey:
		dr = (DialogRecord *)d;
		item = dr->editField+1;
		
		c = (e->message & charCodeMask);
		// BEGIN KLUGE
		if ( c == 27 ) {
			GetDItem(d,2,&iType,&iHandle,&iRect);
			FlashButton((ControlHandle)iHandle);
			*itemHit = 2;
			return (true);
		}
		// END KLUGE
		
		switch (item) {
		case kName:
			if (IsDlgControl(c)) break;
			
			MakeDLOGString(d,e,item,iText);
			if (iText[0] > 12)  // if the name is >12 long ... 
				return AbortEvent(e);
			// make sure every character is legal
			for (i=1; i<=iText[0]; i++) {
				if (! IsPrintableChar(iText[i]))
					return AbortEvent(e);
			}
			break;
		default:
			if (IsDlgControl(c)) break;
			
			maxSize = GetMaxItemSize(item);
			MakeDLOGString(d,e,item,iText);
			
			// make sure every character is legal
			for (i=1; i<=iText[0]; i++) {
				if (! IsNumber(iText[i]))
					return AbortEvent(e);
			}

			StringToNum(iText,(long *)&size);
			if (size > maxSize) {
				switch (OutOfRange(item-3, maxSize)) {
				case 1:
					e->what=nullEvent;
					return false;
					break; // unreachable code;				
				case 2:
				    return (false); // handle normally
				    break; // unreachable code;	
				case 3: // max it out
					NumToString(maxSize,iText);
					GetDItem(d,item,&iType,&iHandle,&iRect);
					SetIText(iHandle,iText);
					SelIText(d,item,0,32767);
					e->what=nullEvent;
					return false;
					break;
				default:
					DebugStr("\pEditFilterProc Error: OutOfRange returned bad item!");
					return;
				} // switch OutOfRange
			} // else continue....
		} // switch (item)
	} // switch (what)
	
	err = GetStdFilterProc(&stdFilterProc);
	if (err == noErr) ret = (stdFilterProc)(d,e,itemHit);
	else ret = false;
	
	return ret;
}


void SetUpEditDialog(DialogPtr d,short id)
{
	short iType,i;
	Handle iHandle;
	Rect iRect;
	Str255 str;
	
	for (i=3; i<34; i++) {
		//if ((i==23) || (i==25)) continue;
		LoadControl(d,i,id);
	}
	
	// Select Name
	GetDItem(d,kName,&iType,&iHandle, &iRect);
	SelIText(d,kName,0,32767);
	
	// Display Cloth quantity
	GetDItem(d,kAQty,&iType,&iHandle, &iRect);
	NumToString((long)AQty[0],str);
	SetIText(iHandle,str);

	// Display Dagger quantity
	GetDItem(d,kWQty,&iType,&iHandle, &iRect);
	NumToString((long)WQty[0],str);
	SetIText(iHandle,str);

}

void CopyEditDialog(DialogPtr d,short id)
{
	short iType,i;
	Handle iHandle;
	Rect iRect;
	Str255 str;
	long num;
	
	// Save top armor value	
	GetDItem(d,kArmor,&iType,&iHandle, &iRect);
	i = GetCtlValue((ControlHandle)iHandle)-1;
	
	GetDItem(d,kAQty,&iType,&iHandle,&iRect);
	GetIText(iHandle,str);
	StringToNum(str,&num);
	AQty[i] = (short) num;

	// Save top weapon value	
	GetDItem(d,kWeapons,&iType,&iHandle, &iRect);
	i = GetCtlValue((ControlHandle)iHandle)-1;
	
	GetDItem(d,kWQty,&iType,&iHandle,&iRect);
	GetIText(iHandle,str);
	StringToNum(str,&num);
	WQty[i] = (short) num;

	// Save out the controls	
	for (i=3; i<34; i++) {
		//if ((i==23) || (i==25)) continue;
		SaveControl(d,i,id);
	}
}

